

# [ bb_html.py ]


import thief_grr.http, thief_grr.html
import bb_sql, bb_http
from bb_conf import *
import HTMLTemplate


# get the HTMLTemplate templates
tmpl = {}
tmpl['generic'] = thief_grr.file2str(rc['base'] + '/html/_generic.html')
tmpl['messenger'] = thief_grr.file2str(rc['base'] + '/html/_messenger.html')
tmpl['index'] = thief_grr.file2str(rc['base'] + '/html/_index.html')
tmpl['thread'] = thief_grr.file2str(rc['base'] + '/html/_thread.html')
tmpl['userlist'] = thief_grr.file2str(rc['base'] + '/html/_userlist.html')
tmpl['register'] = thief_grr.file2str(rc['base'] + '/html/_register.html')
tmpl['login'] = thief_grr.file2str(rc['base'] + '/html/_login.html')
tmpl['profile'] = thief_grr.file2str(rc['base'] + '/html/_profile.html')
tmpl['account'] = thief_grr.file2str(rc['base'] + '/html/_account.html')
tmpl['admin'] = thief_grr.file2str(rc['base'] + '/html/_admin.html')
tmpl['post'] = thief_grr.file2str(rc['base'] + '/html/__post.html')
tmpl['board'] = thief_grr.file2str(rc['base'] + '/html/__board.html')
tmpl['pm'] = thief_grr.file2str(rc['base'] + '/html/__pm.html')


################################
# functions

def bbpath2html(path):
	for i, board in enumerate(path):
		boarddict = thief_grr.dictobj(board)
		path[i] = '''<a class="path" href="index.py?board=%(id)s" title="%(descr)s">%(title)s</a>''' %boarddict
	return '/ ' + ' / '.join(path)

	
################################
# codecs

def lazyEncoder(text):
	return text
def lazyDecoder(text):
	return text
lazyCodecs = (lazyEncoder, lazyDecoder)


################################
# "Types" to be used in Generic

class Message(object):
	"""Both for topmessages and bottommessages."""
	def __init__(self, head=None, body=None, css="box"):
		self.head = head
		self.body = body
		self.css = css
	def html(self):
		h = ''
		if self.head:
			h += '<h1>%s</h1>\n' % self.head
		h += unicode(self.body)
		return h

class Link(object):
	"""For use in the menu."""
	def __init__(self, href=None, content=None, title=None):
		self.href = href
		self.content = content
		self.title = title



################################
# Generic

class Generic(object):
	"""
	All sites, inherit this one.
	"""

	def __init__(self, message=None, css="box"):
		# session
		self.session = bb_http.bbSession()
		self.user = self.session.user
		# content
		self.template = tmpl['messenger']
		self.message = message
		self.css = css
		self.menu = []
		self.menu.append(Link(href="index.py", content="index", title="Top-level boards"))
		self.menu.append(Link(href="userlist.py", content="users", title="Lists registered users"))
		#self.menu.append(Link(href="search.py", content="Search", title="Search the borads."))
		if self.user.isreal:
			self.menu.append(Link(href="pm.py", content="messages", title="Send private messages"))
			self.menu.append(Link(href="account.py", content="account", title="Edit your profile and options"))
			self.menu.append(Link(href="logout.py", content="log out", title="Log out!"))
		else:
			self.menu.append(Link(href="login.py", content="log in", title="Log in!"))
			self.menu.append(Link(href="register.py", content="register", title="Create a new account"))
		self.menu.append(Link(href="html.py?page=help", content="manual", title="RTFM"))
		if rc['mastersite_url']:
			self.menu.append(Link(href=rc['mastersite_url'], content="www", title=rc['mastersite_title']))
		self.leftstatus = "User: <strong>%s</strong>\n<!-- Your bbsessid is %s -->\n" % (self.session.user.username, self.session.bbsessid)
		self.rightstatus = thief_grr.now(human=True)
		self.topmessages = []

		# Check the email
		if self.user.checkemail:
			self.addTop("Please, check your email if you haven't done so since %s.  Something was sent to you then." % thief_grr.unix2human(self.user.checkemail))
			self.user.checkemail = 0.0

		self.bottommessages = []
		self.announcement = None

		# ...Is a real user?
		if self.user.isreal:
			# update lastused
			self.session.update()

	################
	# On-the-fly generated content

	def htmlMain(self):
		t = HTMLTemplate.Template(self.renderMain, self.template, codecs=lazyCodecs)
		return t.render()

	################
	# HTMLTemplate rendering functions

	def renderMain(self, root):
		"""Basically, this is the function that we overide when we inherit from Generic."""
		root.message.content = self.message
		root.message.atts['class'] = self.css

	def renderLink(self, root, link):
		"""Repeater for menu items."""
		try:
			root.a.atts['href'] = link.href
			root.a.content = link.content
			root.a.atts['title'] = link.title
		except:	# No href for this link
			root.a.omit()
			root.content = link.content

	def renderMessage(self, root, message):
		"""Repeater for both topmessages and bottommessages."""
		root.content = message.html()
		root.atts['class'] = message.css

	def renderHtml(self, root):
		"""This renders the generic htmp template."""

		# Are there pending memos for this user?
		for memo in self.user.memos:
			self.addTop(memo, css='memo')
			self.user.rmMemo(memo)
			
		# Are there pending errors for this user?
		for error in self.user.errors:
			self.addTop(error, css='error')
			self.user.rmError(error)

		# top-level stuff
		root.html.atts['xml:lang'] = rc['lang']
		try:
			root.html.head.help.atts['href'] = rc['help']
		except KeyError:
			root.html.head.help.omit()
		root.html.head.title.content = "[%(title)s]: %(subtitle)s" %rc

		w = root.html.body.wrapper

		# header
		w.pagehead.header.title.content = rc['richtitle']
		w.pagehead.header.subtitle.content = rc['subtitle']
		if not rc['subtitle']:
			w.pagehead.header.subtitle.omit()

		# menu
		if self.menu:
			w.pagehead.menu.item.repeat(self.renderLink, self.menu)
		else:
			w.pagehead.menu.content = "Bzzzzz..."

		# status
		if self.leftstatus:
			w.pagehead.status.left.content = self.leftstatus
		if self.rightstatus:
			w.pagehead.status.right.content = self.rightstatus
		# total members
		w.pagehead.status.stats.members.content = str(len(bb_sql.realusers))
		# users online
		countonline = 0
		for user in bb_sql.realusers:
			if user.isonline:
				countonline += 1
		w.pagehead.status.stats.online.content = str(countonline)
		if countonline == 1:
			w.pagehead.status.stats.onlineplural.omit()
		# guests online
		diffguests = bb_sql.Session.select("""user_id = 2 AND lastused > %d""" % (thief_grr.now() - thief_grr.str2secs(rc['max_user_idle'])))
		diffguests = len(list(diffguests))
		w.pagehead.status.stats.guests.content = str(diffguests)
		if diffguests == 1:
			w.pagehead.status.stats.guestsplural.omit()
		
		# total posts
		totalposts = len(list(bb_sql.Post.select()))
		w.pagehead.status.stats.posts.content = str(totalposts)
		# total threads
		totalthreads = len(list(bb_sql.Thread.select()))
		w.pagehead.status.stats.threads.content = str(totalthreads)
		# total boards
		totalboards = len(list(bb_sql.Board.select()))
		w.pagehead.status.stats.boards.content = str(totalboards)

		b = w.pagebody

		# announcement
		if self.announcement:
			b.announcement.body.content = self.announcement
		else:
			b.announcement.omit()
			b.announcementhr.omit()

		# messages
		b.topmessages.topmessage.repeat(self.renderMessage, self.topmessages)
		b.bottommessages.bottommessage.repeat(self.renderMessage, self.bottommessages)
		if not self.topmessages:
			b.topmessageshr.omit()
		if not self.bottommessages:
			b.bottommessageshr.omit()

		# main
		b.main.content = self.htmlMain()
	
	################
	# Compile and final rendering

	def compile(self):
		return HTMLTemplate.Template(self.renderHtml, tmpl['generic'], codecs=lazyCodecs)
	compiled = property(compile)
	
	def render(self):
		return self.compiled.render()
	html = property(render)

	def send(self):
		"""Send all html to stdout (browser)."""
		try:
			thief_grr.http.printHeader(cookies=self.session.httpcookies, charset=rc['charset'])
		except:
			thief_grr.http.printHeader()
		print self.html

	################
	# functions

	def addTop(self, body, head='', css='box'):
		self.topmessages.append(Message(body=body, head=head, css=css))

	def addBottom(self, body, head='', css='box'):
		self.bottommessages.append(Message(body=body, head=head, css=css))



################################
# Index

class ViewBoard(Generic):
	"""
	Sites displayed by index.py, which is basically the board viewer.
	"""

	def __init__(self, board=bb_sql.rootboard, offset=0):
		Generic.__init__(self)
		self.template = tmpl['index']
		self.board = board
		self.offset = max(int(offset), 0)
		self.perpage = int(self.user.options['threads_per_page'])
		self.addTop(unicode(bbpath2html(self.board.path)))
		# which threads should be displayed
		self.threadstodisplay = self.board.threads[self.offset:self.offset+self.perpage]
		# there _must_ be a self.user, even if it's guest
		if self.user.canPostOnBoard(self.board):
			self.addBottom('You can <a href="dbpost.py?board=%d">post</a> on this board.' % self.board.id)
		if self.user.canCreateOnBoard(self.board):
			self.addBottom('You can <a href="dbboard.py?parent=%d">create</a> a new sub-board of %s' % (self.board.id, self.board.title))
		if self.user.canLockBoard(self.board):
			if self.board.locked:
				turnkey = 'unlock'
			else:
				turnkey = 'lock'
			self.addBottom('You can <a href="lock.py?what=board&amp;id=%d">%s</a> this board.' % (self.board.id, turnkey))
		if self.user.canAdministrateBoard(self.board):
			self.addBottom('You can <a href="admin.py?board=%d">administrate</a> this board.' % self.board.id)
		if self.user.canDeleteBoard(self.board):
			self.addBottom('You can <a href="delete.py?what=board&amp;id=%d">delete</a> this board.' % self.board.id)


	def renderThreadRow(self, root, thread):
		if self.user:
			if self.user.canDeleteThread(thread):
				root.thread.x.content = '<a href="delete.py?what=thread&amp;id=%d" title="Delete it"><img src="img/delete.png" alt="delete" /></a>' % thread.id
			if self.user.canLockThread(thread):
				if thread.locked:
					root.thread.l.content = '<a href="lock.py?what=thread&amp;id=%d" title="Unlock it"><img src="img/unlock.png" alt="unlock" /></a>' % thread.id
				else:
					root.thread.l.content = '<a href="lock.py?what=thread&amp;id=%d" title="Lock it"><img src="img/lock.png" alt="lock" /></a>' % thread.id
			# the old/new icons
			if self.user.isvirtual or not thread.lastpost:
				root.thread.status.omit()
			elif thread.lastpost.isNewFor(self.user):
				root.thread.status.atts['src'] = 'img/new.png'
				root.thread.status.atts['alt'] = 'New post'
		root.thread.subject.content = unicode(thread.subject)
		if thread.locked:
			root.thread.locked.content = '(locked!)'
		root.posts.content = unicode(len(thread.posts))
		root.author.content = unicode(thief_grr.html.text2html(thread.author.username, beecode=True, paragraphize=False))
		root.views.content = unicode(thread.views)
		try:
			root.thread.subject.atts['href'] = 'thread.py?thread=%d' % thread.id
			lastpost = thief_grr.dictobj(thread.lastpost)
			root.lastpost.content = '''
			<a class="time" href="thread.py?post=%(id)d#post%(id)d">%(created_human)s</a>,<br />
			by <a class="user" href="profile.py?user=%(author_id)s">%(author_username)s</a>.
			''' %lastpost
		except:	# thread.posts = [] or board.lastpost = False
			root.lastpost.content = "No posts."
	
	def renderChild(self, root, child):
		'''Shortcuts appearing under each board title, if it has subboards.'''
		root.content = unicode(child.title)
		root.atts['href'] = 'index.py?board=%d' % child.id
		root.atts['title'] = unicode(child.descr)

	def renderBoardRow(self, root, board):
		if not self.user.canReadBoard(board):
			root.omit()
		else:
			if self.user.canDeleteBoard(board):
				root.board.x.content = '<a href="delete.py?what=board&amp;id=%d" title="Delete it"><img src="img/delete.png" alt="delete" /></a>' % board.id
			if self.user.canLockBoard(board):
				if board.locked:
					root.board.l.content = '<a href="lock.py?what=board&amp;id=%d" title="Unlock it"><img src="img/unlock.png" alt="unlock" /></a>' % board.id
				else:
					root.board.l.content = '<a href="lock.py?what=board&amp;id=%d" title="Lock it"><img src="img/lock.png" alt="lock" /></a>' % board.id
			# the old/new icons
			if self.user.isvirtual:
				root.board.status.omit()
			# if it's empty, leave the empty.png from the .html gile
			elif board.lastpost:
				if board.lastpost.isNewFor(self.user):
					root.board.status.atts['src'] = 'img/new.png'
					root.board.status.atts['alt'] = 'New posts'
				else:
					root.board.status.atts['src'] = 'img/old.png'
					root.board.status.atts['alt'] = 'Old posts'
	
			root.board.title.atts['title'] = unicode(board.title)
			root.board.title.content = unicode(board.title)
			if board.locked:
				root.board.locked.content = '(locked!)'
			root.board.title.atts['href'] = "index.py?board=%d" % board.id
			if board.descr:
				root.board.descr.content = board.descr
			else:
				root.board.descr.omit()
			# display board's children
			if board.children:
				root.board.shortcuts.children.repeat(self.renderChild, board.children)
			else:
				root.board.shortcuts.omit()
			root.threads.content = unicode(len(board.allthreads))
			root.posts.content = unicode(len(board.allposts))
			try:
				lastpost = thief_grr.dictobj(board.lastpost)
				root.lastpost.content = '''
				<a class="lastpost" href="thread.py?post=%(id)d#post%(id)d">%(subject)s</a>,<br />
				<span class="nobr">on <span class="time">%(created_human)s</span>,</span><br />
				by <a class="user" href="profile.py?user=%(author_id)d">%(author_username)s</a>.
				''' %lastpost
			except AttributeError:	# board.lastpost = False
				root.lastpost.content = "No posts."

	def renderBoardTable(self, root, b):
		if not self.user.canReadBoard(b):
			root.omit()
		else:
			if self.board.threads:
				root.caption.content = "Sub-boards of %s" % self.board.title
			else:
				root.caption.title.atts['href'] = "index.py?board=%d" % b.id
				root.caption.title.content = str(b.title)
				if b.descr:
					root.caption.descr.content = str(b.descr)
				else:
					root.caption.br.omit()
					root.caption.descr.omit()
				if b.locked:
					root.title.content += ' (locked)'
			if b.children:
				root.tbody.subboards.repeat(self.renderBoardRow, b.children)
			else:
				root.thead.omit()
				root.tbody.omit()
				if b == self.board:
					root.omit()

	def renderMain(self, root):
		# If there isn't a subboard table, we won't need the extra seperator
		if not self.board.children:
			root.threadlist.seperator.omit()

		if self.board.threads or self.board.subthreads:
			root.boards.repeat(self.renderBoardTable, [self.board])
			if self.board.threads:
				root.threadlist.title.content = self.board.title
				root.threadlist.threads.repeat(self.renderThreadRow, self.threadstodisplay)
				# navigate
				if len(self.board.threads) > self.perpage:
					r = root.threadlist.navigate
					# prev
					if self.offset >= self.perpage:
						r.left.prev.atts['href'] = 'index.py?board=%d&amp;offset=%d' % (self.board.id, self.offset - self.perpage)
					else:
						r.left.prev.omit()
					# next
					if self.offset + self.perpage <= len(self.board.threads):
						r.right.next.atts['href'] = 'index.py?board=%d&amp;offset=%d' % (self.board.id, self.offset + self.perpage)
					else:
						r.right.next.omit()
					# select
					r.select.option.repeat(self.renderNavigatePage, self.board.threads[::self.perpage])
					# board
					r.board.atts['value'] = str(self.board.id)
				else:
					root.threadlist.navigate.omit()
			else:
				root.threadlist.omit()
		else:
			root.boards.repeat(self.renderBoardTable, self.board.children)
			root.threadlist.omit()

	def renderNavigatePage(self, root, thread):
		threads = self.board.threads
		root.atts['value'] = str(threads.index(thread))
		root.content = str(threads[::self.perpage].index(thread) + 1)
		if self.offset == threads.index(thread):
			root.atts['selected'] = 'selected'


################################
# Thread

class ViewThread(Generic):
	"""
	Thread viewer, run via thread.py.
	"""

	def __init__(self, thread=None, order="asc", offset=0, post=None):
		Generic.__init__(self)
		self.template = tmpl['thread']
		self.perpage = int(self.user.options['posts_per_page'])
		self.thread = thread
		self.post = post
		self.offset = max(int(offset), 0)
		if self.post:
			self.thread = self.post.thread
			index = self.thread.posts.index(self.post)
			self.offset = index - (index % self.perpage)
		self.board = self.thread.board
		self.addTop(unicode(bbpath2html(self.board.path)))
		if self.user.canPostOnThread(self.thread):
			self.addBottom('You can <a href="dbpost.py?thread=%d">post a reply</a> on this thread.' % self.thread.id)
		# which posts should be displayed
		self.poststodisplay = self.thread.posts[self.offset:self.offset+self.perpage]
		# increment the views
		if self.thread:
			self.thread.views += 1


	def renderPost(self, root, post):
		# mark as read
		if self.user.isreal and post.created > self.user.lastvisit:
			self.session.s.addPost(post)
		# highlight post
		if self.post is post:
			root.atts['class'] += ' highlight'
		# display post
		root.atts['id'] = "post%d" % post.id
		root.head.left.content = post.created_human
		root.head.right.head.content = thief_grr.html.text2html(post.head, beecode=True, paragraphize=False)
		root.body.left.username.content = post.author.username
		root.body.left.username.atts['href'] = '''profile.py?user=%d''' % post.author.id
		if post.author.details and post.author.details.avatar:
			root.body.left.avatar.atts['src'] = 'avatars/' + post.author.details.avatar
			root.body.left.avatar.atts['alt'] = '''Avatar of %s''' % post.author.username
		else:
			root.body.left.avatar.omit()
		# full name
		if post.author.fullname:
			root.body.left.fullname.content = post.author.fullname
		else:
			root.body.left.fullname.omit()
		root.body.right.text.content = thief_grr.html.text2html(post.body, beecode=True)
		# signature
		if post.author.details and post.author.details.signature:
			root.body.right.signature.content = thief_grr.html.text2html(post.author.details.signature, beecode=True)
		else:
			root.body.right.signature.omit()
			root.body.right.signaturehr.omit()
		# online / offline
		if post.author.isonline:
			root.feet.left.img.atts['src'] = "img/online.png"
			root.feet.left.img.atts['alt'] = "*"
			root.feet.left.status.content = "Online"
		# message
		if self.user.canMessage(post.author):
			root.feet.left.m.content = '''<a href="pm.py?to=%s" title="Send %s a message"><img src="img/message.png" alt="msg" /></a>''' % (post.author.username, post.author.username)
		# last updated
		if post.updated:
			root.feet.right.updated.content = post.updated_human
		else:
			root.feet.right.updated.omit()
		# post url
		posturl = 'http://%s/thread.py?post=%d#post%d' % (rc['url'], post.id, post.id)
		root.feet.right.posturl.atts['href'] = posturl
		# delete post?
		if self.user.canDeletePost(post):
			root.head.right.x.content = '<a href="delete.py?what=post&amp;id=%d" title="Delete post"><img src="img/delete.png" alt="delete" /></a>' % post.id
		# edit post?
		if self.user.canEditPost(post):
			root.head.right.e.content = '<a href="dbpost.py?post=%d" title="Edit post"><img src="img/edit.png" alt="edit" /></a>' % post.id
		
	def renderMain(self, root):

		# posts
		root.post.repeat(self.renderPost, self.poststodisplay)

		# quick reply
		if not self.thread.posts or self.thread.lastpost in self.poststodisplay and self.user.canPostOnThread(self.thread):
			r = root.quickreply
			r.head.left.content = thief_grr.now(human=True)
			r.head.right.replyhead.atts['value'] = unicode(self.thread.lastpost.head)
			r.body.left.username.content = self.user.username
			r.body.left.username.atts['href'] = '''profile.py?user=%d''' % self.user.id
			if self.user.details and self.user.details.avatar:
				r.body.left.avatar.atts['src'] = 'avatars/' + self.user.details.avatar
				r.body.left.avatar.atts['alt'] = '''Avatar of %s''' % self.user.username
			else:
				r.body.left.avatar.omit()
			# full name
			if self.user.fullname:
				r.body.left.fullname.content = self.user.fullname
			else:
				r.body.left.fullname.omit()
			# signature
			if self.user.details and self.user.details.signature:
				r.body.right.signature.content = self.user.details.signature
			else:
				r.body.right.signature.omit()
				r.body.right.signaturehr.omit()
			r.thread.atts['value'] = str(self.thread.id)
		else:
			root.quickreply.omit()

		# navigate
		if len(self.thread.posts) > self.perpage:
			r = root.navigate
			# prev
			if self.offset >= self.perpage:
				r.left.prev.atts['href'] = 'thread.py?thread=%d&amp;offset=%d' % (self.thread.id, self.offset - self.perpage)
			else:
				r.left.prev.omit()
			# next
			if self.offset + self.perpage < len(self.thread.posts):
				r.right.next.atts['href'] = 'thread.py?thread=%d&amp;offset=%d' % (self.thread.id, self.offset + self.perpage)
			else:
				r.right.next.omit()
			# select
			r.select.option.repeat(self.renderNavigatePage, self.thread.posts[::self.perpage])
			# thread
			r.thread.atts['value'] = str(self.thread.id)
		else:
			root.navigate.omit()

	def renderNavigatePage(self, root, post):
		posts = self.thread.posts
		root.atts['value'] = str(posts.index(post))
		root.content = str(posts[::self.perpage].index(post) + 1)
		if self.offset == posts.index(post):
			root.atts['selected'] = 'selected'




################################
# Userlist

class ViewUserlist(Generic):
	"""
	Displays a list of registered users, and brief info about them.
	Run via userlist.py.
	"""

	def __init__(self, offset=0, order="asc", orderby="username"):
		Generic.__init__(self)
		self.offset = max(int(offset), 0)
		self.template = tmpl['userlist']
		self.orderby = orderby
		self.order = order
		self.users = list(bb_sql.User.select(orderBy=self.orderby))
		self.offset = max(int(offset), 0)
		self.perpage = int(self.user.options['users_per_page'])
		# which users should be displayed
		self.userstodisplay = self.users[self.offset:self.offset+self.perpage]

	def renderUser(self, root, user):
		if user.isreal:
			userdict = thief_grr.dictobj(user)
			if user.isonline:
				root.status.atts['src'] = 'img/online.png'
				root.status.atts['alt'] = 'online'
				root.lastvisit.content = 'Online now'
				root.atts['class'] = 'online'
			else:
				root.status.atts['src'] = 'img/offline.png'
				root.status.atts['alt'] = 'offline'
				root.lastvisit.content = user.lastvisit_human
			if self.user.canMessage(user) and user is not self.user:
				root.m.content = '''<a href="pm.py?to=%s" title="Send %s a message"><img src="img/message.png"   alt="msg" /></a>''' % (user.username, user.username)
			else:
				root.m.omit()
			root.username.atts['href'] = 'profile.py?user=%(id)d' %userdict
			root.username.atts['title'] = 'View the profile of %(username)s' %userdict
			root.username.content = user.username
			root.fullname.content = user.fullname
			root.posts.content = unicode(len(user.posts))
			root.created.content = user.created_human
		
	def renderMain(self, root):
		root.userlist.title.content = 'Userlist'
		root.userlist.user.repeat(self.renderUser, self.userstodisplay)

		# navigate
		if len(self.users) > self.perpage:
			r = root.navigate
			# prev
			if self.offset >= self.perpage:
				r.left.prev.atts['href'] = 'userlist.py?offset=%d' % (self.offset - self.perpage)
			else:
				r.left.prev.omit()
			# next
			if self.offset + self.perpage < len(self.users):
				r.right.next.atts['href'] = 'userlist.py?offset=%d' % (self.offset + self.perpage)
			else:
				r.right.next.omit()
			# select
			r.select.option.repeat(self.renderNavigatePage, self.users[::self.perpage])
		else:
			root.navigate.omit()

	def renderNavigatePage(self, root, user):
		users = self.users
		root.atts['value'] = str(users.index(user))
		root.content = str(users[::self.perpage].index(user) + 1)
		if self.offset == users.index(user):
			root.atts['selected'] = 'selected'



################################
# Register

class Register(Generic):
	"""
	Displays the registration form, for new users.
	"""

	def __init__(self, POST=None):
		Generic.__init__(self)
		self.template = tmpl['register']
		self.POST = POST

	def renderMain(self, root):
		root.form.ip.content = thief_grr.http.getIP()
		root.form.time.content = thief_grr.now(human=True)


################################
# Login

class Login(Generic):
	"""
	Displays the login form.
	"""

	def __init__(self):
		Generic.__init__(self)
		self.template = tmpl['login']

	def renderMain(self, root):
		root.form.ip.content = thief_grr.http.getIP()


################################
# Profile

class Profile(Generic):
	"""
	Displays a user's profile.
	"""

	def __init__(self, showuser):
		Generic.__init__(self)
		self.template = tmpl['profile']
		self.showuser = showuser
		if self.user is self.showuser:
			self.addBottom('''You can <a href="account.py">edit</a> your profile.''')
	
	def renderMain(self, root):
		root.username.content = self.showuser.username
		if self.showuser.details:
			root.fullname.content = self.showuser.fullname
			root.location.content = self.showuser.details.location
			root.website.content = self.showuser.details.website
			if self.showuser.details.publicemail:
				root.publicemail.content = self.showuser.details.publicemail
			if self.showuser.details.about:
				root.about.about.content = self.showuser.details.about
			else:
				root.about.omit()
			if self.showuser.details.photo:
				root.photo.atts['src'] = 'photos/' + self.showuser.details.photo
			else:
				root.photo.omit()
		else:
			root.photo.omit()
			root.about.omit()
		root.stats.totalposts.content = str(len(self.showuser.posts))
		root.stats.totalthreads.content = str(len(self.showuser.threads))


################################
# Account

class Account(Generic):
	"""
	Editing a user's account.
	"""

	def __init__(self):
		Generic.__init__(self)
		self.template = tmpl['account']
		self.user = self.session.user
	
	def renderMain(self, root):
		# User
		root.user.username.content = self.user.username
		root.user.level.content = unicode(self.user.level)
		root.user.email.atts['value'] = self.user.email
		# Options
		if self.user.options['accept_messages']:
			root.options.accept_messages.atts['checked'] = 'checked'
		# Details
		if self.user.details:
			root.details.firstname.atts['value'] = self.user.details.firstname
			root.details.lastname.atts['value'] = self.user.details.lastname
			root.details.photo.atts['value'] = self.user.details.photo
			root.details.location.atts['value'] = self.user.details.location
			root.details.about.content = self.user.details.about
			root.details.website.atts['value'] = self.user.details.website
			root.details.publicemail.atts['value'] = self.user.details.publicemail
			root.details.signature.content = self.user.details.signature
			root.details.avatar.atts['value'] = self.user.details.avatar


################################
# Admin

class Admin(Generic):
	"""
	Performing administrative tasks for boards.
	"""

	def __init__(self, board):
		Generic.__init__(self)
		self.template = tmpl['admin']
		self.user = self.session.user
		self.board = board
	
	def renderUsername(self, root, user):
		root.content = user.username

	def renderOption(self, root, user):
		root.content = user.username
		root.atts["value"] = user.username

	def renderMain(self, root):
		root.title.atts['value'] = self.board.title
		root.descr.content = str(self.board.descr)
		root.boardid.atts['value'] = unicode(self.board.id)
		root.rlevel.atts['value'] = unicode(self.board.rlevel)
		root.wlevel.atts['value'] = unicode(self.board.wlevel)
		root.prec.atts['value'] = unicode(self.board.prec)
		root.admins.admin.repeat(self.renderUsername, self.board.admins)
		root.moderator.repeat(self.renderOption, self.board.moderators)
		peasants = list(bb_sql.User.select())
		# get rid of non-peasants
		for np in set(self.board.admins).union(set(self.board.moderators)).union(set(bb_sql.virtualusers)):
			peasants.remove(np)
		root.peasant.repeat(self.renderOption, peasants)


################################
# Dbpost

class Dbpost(Generic):
	"""
	Posting new posts.
	"""

	def __init__(self, board=None, thread=None, post=None):
		Generic.__init__(self)
		self.template = tmpl['post']
		self.user = self.session.user
		self.board = board
		self.thread = thread
		self.post = post
	
	def renderMain(self, root):
		# board
		if not self.board:
			root.form.board.omit()
		else:
			root.title.content = "New thread"
			root.form.board.atts['value'] = unicode(self.board.id)
			root.form.reset.atts['value'] = "Clear"
			root.form.submit.atts['value'] = "Post"
		# thread
		if not self.thread:
			root.form.thread.omit()
		else:
			root.title.content = "New post"
			root.form.thread.atts['value'] = unicode(self.thread.id)
			root.form.reset.atts['value'] = "Clear"
			root.form.submit.atts['value'] = "Reply"
		# post
		if not self.post:
			root.form.post.omit()
		else:
			root.title.content = "Edit post"
			root.form.caption.content = self.post.head
			root.form.head.atts['value'] = self.post.head
			root.form.body.content = self.post.body
			root.form.post.atts['value'] = str(self.post.id)
			root.form.reset.atts['value'] = "Undo"
			root.form.submit.atts['value'] = "Apply"


################################
# Dbboard

class Dbboard(Generic):
	"""
	Creating new boards.
	"""

	def __init__(self, parent=None):
		Generic.__init__(self)
		self.template = tmpl['board']
		self.parent = parent
	
	def renderMain(self, root):
		root.title.content = "New board under %s" % self.parent.title
		if not self.parent:
			root.form.parent.omit()
		else:
			root.form.parent.atts['value'] = unicode(self.parent.id)
		root.form.rlevel.atts['value'] = unicode(self.parent.rlevel)
		root.form.wlevel.atts['value'] = unicode(self.parent.wlevel)



################################
# PM

class PM(Generic):
	"""
	Sending private messages.
	"""

	def __init__(self, to=None):
		Generic.__init__(self)
		self.template = tmpl['pm']
		self.to = to
		if self.to and not self.to.options['accept_messages']:
			self.addTop('''User <a href="userlist.py?user=%d" title="View user's profile">%s</a> doesn't accept messages.''' % (self.to.id, self.to.username))
		if self.to == self.user:
			self.addTop('''...You <em>lunatic</em>...''')
	
	def renderMain(self, root):
		root.title.content = "%s messenger" % rc['title']
		if self.to:
			if self.to.options['accept_messages']:
				root.form.to.atts['value'] = self.to.username
				self.addTop("User %s doesn't accept messages." % self.to.username)


