

# [ bb_http.py ]


import cPickle, os, sys, time
import thief_grr.http, bb_sql
import bb_html


def newSession():
	"""
	Create a random string which does not already exist.
	Seriously, it won't ever exist.
	"""

	r = thief_grr.randstr()
	while list(bb_sql.Session.selectBy(bbsessid=r)):
		r = thief_grr.randstr()
	return r


class cookieWrapper(object):
	"""
	This is a dictionary-like object, for use by bbSession.
	"""

	def __init__(self, bs):
		self.bs = bs
	
	def __setitem__(self, cookie, value):
		self.bs.setCookie(cookie, value)

	def __getitem__(self, cookie):
		return self.bs.getCookie(cookie)


class bbSession(object):
	"""
	Session implementation.
	"""

	def __init__(self):
		# New session or not?
		try: # old
			self.bbsessid = thief_grr.http.getCOOKIES()['beebb']
			self.s = bb_sql.Session.byBbsessid(self.bbsessid)	# SQLObject
			self.update()
		except: # new
			self.bbsessid = newSession()
			self.s = bb_sql.Session(bbsessid=self.bbsessid, user=bb_sql.guest)
		# Cookie jar
		self.jar = cookieWrapper(self)

	def update(self):
		self.s.lastused = int(time.time())

	def terminate(self):
		self.s.terminate()

	# handle sql cookies

	def getCookie(self, cookie):
		"""Gets the cookie from the db"""
		c = cPickle.loads(self.s.cookies)
		try:
			return c[cookie]
		except KeyError:
			return None
	
	def setCookie(self, cookie, value):
		"""Updates the db with the new cookie"""
		c = cPickle.loads(self.s.cookies)
		if not c:	# no cookies yet?
			c = {}	# not anymore...
		c[cookie] = unicode(value)
		self.s.cookies = cPickle.dumps(c)

	# properties

	def httpCookies(self):
		"""The only real, actual HTTP cookie is this one..."""
		return {"beebb": self.bbsessid}

	def getUser(self):
		try:
			return self.s.user
			#return bb_sql.User.byUsername(self.getCookie("username"))
		except Exception, e:
			return bb_sql.guest

	httpcookies = property(httpCookies)
	user = property(getUser)


def membersonly():
	"""
	Should be called, first-thing, on members-only pages.
	It returns the current user, if one exists.
	"""

	user = bbSession().user
	if user == bb_sql.guest:
		website = bb_html.Generic(message="This page is for members only.")
		website.send()
		sys.exit(0)
	else:
		return user

def guestsonly():
	"""
	Should be called, first-thing, on guests-only pages.
	"""

	user = bbSession().user
	if not user == bb_sql.guest:
		website = bb_html.Generic(message="This page is for guests only.")
		website.send()
		sys.exit()

def must(b, message="Access denied.  Ha!"):
	"""
	Should be called, first-thing, on a page, along with a condition b.
	If b is True, it does nothing.  Otherwise it displays a permission denied
	website with message for text.
	"""

	if not b:
		website = bb_html.Generic(message=message)
		website.send()
		sys.exit(0)
