

# [ bb_image.py ]


import Image, urllib
from bb_conf import rc
from bb_error import BeeBBError
from os import stat
from thief_grr import str2bytes


def fetchImage(urlstring, directory, filename, max_size):
	"""
	Wrapper of urlretrieve.
	"""

	if urlstring.find('http://') == 0:
		def reporthook(block_count, block_size, file_size):
			if file_size > max_size:
				raise BeeBBError, "File size %d > maximum size %d" % (file_size, max_size)
		return urllib.urlretrieve(urlstring, filename='%s/%s' % (directory, filename), reporthook=reporthook)
	else:
		return '%s/%s' % (directory, urlstring), 'Using stored image.'


def isValid(f, max_size=0, max_width=0, max_height=0):
	"""
	Valdates imagefile f to check if it is an accepted Image.
	If it is, it returns True.  Otherwise, an exception is raised.
	"""

	img = Image.open(f)
	file_size = stat(f)[6]
	if file_size > max_size:
		raise BeeBBError, "File size %d > maximum size %d" % (file_size, max_size)
	elif img.size[0] > max_width:
		raise BeeBBError, "Image width %d > maximum width %d" % (img.size[0], max_width)
	elif img.size[1] > max_height:
		raise BeeBBerror, "Image height %d > maximum height %d" % (img.size[1], max_height)
	else:
		return True


def isPhoto(f):
	"""
	Valdates Photos.
	"""

	constraints = {}
	constraints['max_size'] = str2bytes(rc['max_photo_size'])
	constraints['max_width'] = int(rc['max_photo_width'])
	constraints['max_height'] = int(rc['max_photo_height'])
	return isValid(f, **constraints)


def isAvatar(f):
	"""
	Valdates Avatars.
	"""

	constraints = {}
	constraints['max_size'] = str2bytes(rc['max_avatar_size'])
	constraints['max_width'] = int(rc['max_avatar_width'])
	constraints['max_height'] = int(rc['max_avatar_height'])
	return isValid(f, **constraints)
