

# [ bb_mail.py ]


from bb_conf import rc
from thief_grr.smtp import mail as thief_mail
from string import Template
from thief_grr import file2str, now
from thief_grr.http import getIP




def mail(to_addr, from_addr="%(title)s Services <%(services_email)s>"%rc, subject='', body=''):
	"""
	Send an email using the smtpd specified.
	"""

	thief_mail(server=rc['smtpd'], from_addr=from_addr, to_addr=to_addr, subject=subject, body=body)


def TrialUser(to_addr, key):
	"""
	Send a confirmation email to the user trying to register.
	"""
	
	# body
	confirmationurl = 'https://%s/confirm.py?what=user&key=%s' % (rc['url'], key)
	bodytmpl = Template(file2str(rc['base'] + '/msg/trialuser.email'))
	body = bodytmpl.substitute(rc, confirmationurl=confirmationurl, ip=getIP())

	# headers
	subject = "%(title)s:  Registration confirmation needed" %rc

	# send
	mail(to_addr=to_addr, subject=subject, body=body)


def TrialEmail(to_addr, key, user):
	"""
	Send a confirmation email to the user trying to change their email address.
	"""
	
	# body
	confirmationurl = 'https://%s/confirm.py?what=email&key=%s' % (rc['url'], key)
	bodytmpl = Template(file2str(rc['base'] + '/msg/trialemail.email'))
	body = bodytmpl.substitute(rc, confirmationurl=confirmationurl, ip=getIP(), old=user.email, new=to_addr, user=user.username)

	# headers
	subject = "%(title)s:  Email change confirmation needed" %rc

	# send
	mail(to_addr=to_addr, subject=subject, body=body)


def TrialReset(to_addr, key):
	"""
	Send a confirmation email to the user trying to reset their passwd.
	"""

	# body
	confirmationurl = 'https://%s/confirm.py?what=reset&key=%s' % (rc['url'], key)
	bodytmpl = Template(file2str(rc['base'] + '/msg/trialreset.email'))
	body = bodytmpl.substitute(rc, confirmationurl=confirmationurl, ip=getIP())

	# headers
	subject = "%(title)s:  Password reset confirmation" %rc

	# send
	mail(to_addr=to_addr, subject=subject, body=body)


def pm(sender, recepient, message):
	"""
	Send an email to recepient.
	"""

	if recepient.fullname:
		recepient_name = recepient.fullname
	else:
		recepient_name = recepient.username

	# body
	replyurl = 'http://%s/pm.py?to=%s' % (rc['url'], sender.username)
	bodytmpl = Template(file2str(rc['base'] + '/msg/pm.email'))
	body = bodytmpl.substitute(rc, replyurl=replyurl, ip=getIP(), message=message, sender=sender.username, recepient=recepient_name)

	# headers
	if sender.options['accept_messages']:
		reply_to = sender.email
	else:
		reply_to = rc['services_email']
	from_addr = "%s from %s <%s>" % (sender.username, rc['title'], reply_to)
	subject = "[%s] Message from %s" % (rc['title'], sender.username)

	# send
	mail(from_addr=from_addr, to_addr=recepient.email, subject=subject, body=body)
	recepient.checkemail = now()
